diff -ru ZynAddSubFX-2.2.1/src/Makefile.inc ZynAddSubFX-2.2.1-jackmidi/src/Makefile.inc
--- ZynAddSubFX-2.2.1/src/Makefile.inc	2005-04-28 02:21:35.000000000 +0200
+++ ZynAddSubFX-2.2.1-jackmidi/src/Makefile.inc	2005-10-02 15:09:53.000000000 +0200
@@ -29,9 +29,9 @@
 #parameters (run 'zynaddsubfx --help' for help) 
 
 #LINUX_AUDIOOUT=OSS_AND_JACK
-LINUX_AUDIOOUT=OSS
+#LINUX_AUDIOOUT=OSS
 #LINUX_AUDIOOUT=NONE
-#LINUX_AUDIOOUT=JACK
+LINUX_AUDIOOUT=JACK
 #LINUX_AUDIOOUT=JACK_RT     JACK_RT support is broken
 #for PortAudio (PA)
 #LINUX_AUDIOOUT=PA
diff -ru ZynAddSubFX-2.2.1/src/Output/JACKaudiooutput.C ZynAddSubFX-2.2.1-jackmidi/src/Output/JACKaudiooutput.C
--- ZynAddSubFX-2.2.1/src/Output/JACKaudiooutput.C	2005-03-12 19:16:26.000000000 +0100
+++ ZynAddSubFX-2.2.1-jackmidi/src/Output/JACKaudiooutput.C	2005-10-02 15:08:48.000000000 +0200
@@ -21,11 +21,14 @@
 */
 
 #include <stdlib.h>
+extern "C" {
+#include <jack/midiport.h>
+}
 #include "JACKaudiooutput.h"
 
 Master *jackmaster;
 jack_client_t *jackclient;
-jack_port_t *outport_left,*outport_right;
+jack_port_t *outport_left,*outport_right,*midi_inport;
 
 int jackprocess(jack_nframes_t nframes,void *arg);
 int jacksrate(jack_nframes_t nframes,void *arg);
@@ -60,6 +63,8 @@
 	JACK_DEFAULT_AUDIO_TYPE,JackPortIsOutput|JackPortIsTerminal,0);    
     outport_right=jack_port_register(jackclient,"out_2",
 	JACK_DEFAULT_AUDIO_TYPE,JackPortIsOutput|JackPortIsTerminal,0);    
+    midi_inport=jack_port_register(jackclient,"midi_input",
+	JACK_DEFAULT_MIDI_TYPE,JackPortIsInput|JackPortIsTerminal,0);    
 
     if (jack_activate(jackclient)){
 	fprintf(stderr,"Cannot activate jack client\n");
@@ -78,6 +83,7 @@
     jack_default_audio_sample_t *outr=(jack_default_audio_sample_t *) jack_port_get_buffer (outport_right, nframes);
 
     pthread_mutex_lock(&jackmaster->mutex);
+    JACKhandlemidi(nframes);
     jackmaster->GetAudioOutSamples(nframes,jack_get_sample_rate(jackclient),outl,outr);
     pthread_mutex_unlock(&jackmaster->mutex);
     
@@ -97,4 +103,51 @@
 };
 
 
+void JACKhandlemidi(unsigned long frames) {
+  
+  // We must have the master mutex before we run this function
+  
+  // XXX This is really nasty, not only do we lose the sample accuracy of
+  // JACK MIDI, but any accuracy at all below the buffer size
+  
+  void* midi_buf = jack_port_get_buffer(midi_inport, frames);
+  jack_midi_event_t jack_midi_event;
+  jack_nframes_t event_index = 0;
+  jack_nframes_t event_count = 
+    jack_midi_port_get_info(midi_buf, frames)->event_count;
+  unsigned char* midi_data;
+  unsigned char type, chan;
+  
+  while (event_index < event_count) {
+    
+    jack_midi_event_get(&jack_midi_event, midi_buf, event_index, frames);
+    midi_data = jack_midi_event.buffer;
+    type = midi_data[0] & 0xF0;
+    chan = midi_data[0] & 0x0F;
+    
+    switch (type) {
+
+    case 0x80: /* note-off */
+      jackmaster->NoteOff(chan, midi_data[1]);
+      break;
+      
+    case 0x90: /* note-on */
+      jackmaster->NoteOn(chan, midi_data[1], midi_data[2]);
+      break;
+    
+    case 0xB0: /* controller */
+      jackmaster->SetController(chan, midi_data[1], midi_data[2]);
+      break;
+    
+    case 0xE0: /* pitch bend */
+      jackmaster->SetController(chan, C_pitchwheel,
+				((midi_data[2] << 7) | midi_data[1]));
+      break;
 
+    /* XXX TODO: handle MSB/LSB controllers and RPNs and NRPNs */
+    }    
+    
+    event_index++;
+  }
+  
+}
diff -ru ZynAddSubFX-2.2.1/src/Output/JACKaudiooutput.h ZynAddSubFX-2.2.1-jackmidi/src/Output/JACKaudiooutput.h
--- ZynAddSubFX-2.2.1/src/Output/JACKaudiooutput.h	2005-03-12 19:16:26.000000000 +0100
+++ ZynAddSubFX-2.2.1-jackmidi/src/Output/JACKaudiooutput.h	2005-10-02 14:40:05.000000000 +0200
@@ -40,6 +40,7 @@
 
 bool JACKaudiooutputinit(Master *master_);
 void JACKfinish();
+void JACKhandlemidi(unsigned long frames);
 
 #endif
 
